/*
	$Id: list.h,v 3.2.2.2 2011/09/29 11:50:50 shelton Exp $
*/
#ifndef LIST_H
#define LIST_H

#ifdef DJGPP

#ifndef u_char
#define u_char unsigned char
#endif

#ifndef u_short
#define u_short unsigned short
#endif

#ifndef u_long
#define u_long unsigned long
#endif

#endif

class TList
{
protected:
  void *Head;
  void *Current;

  // defined by object type in list
  virtual void *getNext(void *item) = 0;
  virtual void *getPrev(void *item) = 0;
  virtual void setNext(void *item, void *next) = 0;
  virtual void setPrev(void *item, void *prev) = 0;
  virtual void freeItem(void *item) = 0;

public:
  TList():Head(0),Current(0) {;}
  virtual ~TList() {;}

  // at subsequent class you should define somewhat, like
  // Type *item() { return (Type*) getCurrent(); }
  // for access to element with types extension
  void *getCurrent() { return Current; }
  int empty() {return Head==0;}

  int  first();
  int  last();
  int  next();
  void  Next(); // ring next
  int  prev();
  void  Prev(); // ring prev
  int  seek(void *item);   // sets current

  // These couple of functions do not check presence in list
  // and must use only when you clearly apologize, that
  // item is part of list
  void setCurrent(void *item)	{ Current=item; }
  void removeIt(void *item);
  void freeIt(void *item);

  void insert(void *item);       // insert after current
  void insertBefore(void *item); // insert before current
  void append(void *item);       // insert at end
  void prepend(void *item);      // insert at begin

  int  remove(void *item);
  void remove();           // delete current; now current is next
  void removeAll();
  int  free_item(void *item);
  void free_cur();             // free current
  void freeAll();

  void put(void *item);     // replace current
  void putFree(void *item); // replace current and free it
};

#endif
