// $Id: term.h,v 3.2.2.2 2011/09/29 11:50:50 shelton Exp $
#ifndef _TERM_H_
#define _TERM_H_

#define BASE_TERMCAP	0
#define BASE_TERMINFO	1

#define CAPFLG	0
#define CAPNUM	1
#define CAPSTR	2

#define VisualBold	1
#define VisualDim	2
#define VisualInverse	4
#define VisualColors	8
#define VisualGraph	16

typedef struct {
	char *tcap;
	char type;	/* CAPFLG, CAPNUM, CAPSTR */
	char *val_bool;
	int  *val_int;
	char **val_ptr;
} CapMap;

typedef struct {
	char *tcap;
	char *str;
	long val;
} KeyMap;

class Term
{
public:
	Term() {}
	~Term() {}
	static char *tname;
	static int rows, cols;
	static short ctab[16], btab[16];
	static unsigned short linedraw[11];

	static char *BS, *BR, *DS, *DR, *NS, *NR;
	static char *GS, *GE, *G1, *G2, *GT, *AC, *AS, *AE;
	static char *CS, *SF, *SR, *EA;

	static char *KS, *KE;

	static char *CL, *CE, *CM, *SE, *SO, *TE, *TI, *VE, *VS,
		*AL, *DL, *IS, *IF, *FS, *MD, *MH, *ME, *MR,
		*CF, *CB, *AF, *AB, *Sf, *Sb, *MF, *MB, *OP;
	static int NF, NB;
	static char MS, C2;

	static char Cy;
	static char *Cs, *Ce, *Ct;

	static int Visuals;
	static int VisualMask;
	static int scrool, rscrool;
	static CapMap captab[];
	static KeyMap keytab[];

	static int init( int base );
	static void destroy();
	static char *makecolor( int c, int b );
	static void initbold();
	static void initgraph();
	static void initcolor( char *fg, char *bg );

	static int hasDim();
	static int hasBold();
	static int hasInverse();
	static int hasColors();
	static int hasGraph();

	/* stndard enter mode */
	static unsigned char inputTable[256];
	/* national enter mode (translation of Latin-1 symbols */
	static unsigned char cyrInputTable[256];
	static char cyrOutputTable[256];

	static void sysvgoto( register char *outp, register char *cp, int p1, int p2 );
	static char *Goto (char *cm, int destcol, int destline);

	static void SetTty();
	static void ResetTty();
	static void fill_info();
};

/*
 *	Symbols 0 - 127 	assumed as Latin-1
 *		128 - 191 	assumed as pseudographic
 *		192 - 255	assumed as national
 *	This structure is equal with koi8 codetable, but at all
 *	allows describe any alphabet with symbols quantity <= 32
 *
 *	Structure of description file for transtaling chars (termcap entry Ct):
 *
 *	64 chars - translation national chars at output
 *	96 chars - translation Latin-1 chars (32 - 127) at national at input
 *	128 chars - translation up-half table at pseudographic or
 *		    national chars at input
 *	64 chars - translation pseudographic chars at output
 *
 *	- structure is compatible "down" with deco-usable
 *
 *	alghoritm for output:
 *	for national char :
 *	if Cs != 0, output Cs for switching terminal
 *	at national output mode and Ce to switching back
 *	for pseudographic char :
 *	if gs != 0, output gs for switching terminal at \
 *	pseudographic output mode and ge for switching back
 *
 *	for n char output outputTable[n]
 *
 */

/*
 * linedraw array contains chars for border drawing:
 *     0    horisontal line
 *     1    vertical line
 *     2    lower left corner
 *     3    lower center
 *     4    lower right corner
 *     5    left center
 *     6    center cross
 *     7    right center
 *     8    upper left corner
 *     9    upper center
 *     10   upper right corner
 */

#ifdef SCO_SYSV
  #ifndef _STRUCT_WINSIZE
    #define _STRUCT_WINSIZE
struct winsize {
	unsigned short ws_row;		/* rows, in characters */
	unsigned short ws_col;		/* columns, in character */
	unsigned short ws_xpixel;	/* horizontal size, pixels */
	unsigned short ws_ypixel;	/* vertical size, pixels */
};
  #endif

#endif

#endif /* _TERM_H_ */
