/*
	$Id: termcap.h,v 3.2.2.2 2011/09/29 11:50:50 shelton Exp $
*/
#ifndef _TERMCAP_H_
#define _TERMCAP_H_

/*
const int CAPNUM = 1;
const int CAPFLG = 2;
const int CAPSTR = 3;
*/
#define CAPNUM	1
#define CAPFLG	2
#define CAPSTR	3

struct Captab {
	char tname [3];
	char ttype;
	char tdef;
	char *tc;
	int *ti;
	char **ts;
};

struct Keytab {
	char *tcap;
	char *str;
	long val;
};


class Termcap
{
   friend class Screen;
   static int init;
   static int initTermcap();

public:
   static char *term;
   static Keytab keytab[];

   static char MS, C2;
   static int  NF, NB, Lines, Columns;
   static unsigned short linedraw [11];

   // data about terminal
   static int hasDim();
   static int hasBold();
   static int hasInverse();
   static int hasColors();
   static int hasGraph();

   static void setColorMap(char * fg, char *bg);
   static short ctab [16], btab [16]; // color of char/back translation tables
   static int scrool, rscrool;


   /* for output in national mode (translation Latin-1 chars) */
   static unsigned char cyrInputTable [256];
   /* standard mode input */
   static unsigned char inputTable  [256];
   /* output */
   static unsigned char outputTable [256];

   // terminal control commands
   static char *AS, *AE, *AC, *GS, *GE, *G1, *G2, *GT;
   static char *CS, *SF, *SR;
   static char *PO, *PF, *POO;
   static char *CL, *CE, *CM, *SE, *SO, *TE, *TI, *VI, *VE, *VS;
   static char *AL, *DL, *FS, *MD, *MH, *ME, *MR;
   static char *CF, *CB, *MF, *MB, *OP;

   static char *MsH, *MsL;

   static char *KS, *KE;

   static char Cy; // terminal accepted Cyrillic chars
   static char *Cs, *Ce,
	  *Ct;    // translation table filename

   static char *Goto (char *cm, int destcol, int destline);

   static int Visuals;      // video enchancement mask
   static int VisualMask;   // allowed attributes mask

   static void SetTty();
   static void ResetTty();

};

/*
const int VisualBold = 1;    // high bright is there (md)
const int VisualDim  = 2;    // low bright is there (mh)
const int VisualInverse = 4;    // is there inversion (so or mr)
const int VisualColors  = 8;    // is there colors (NF, NB, CF, CB, C2, MF, MB)
const int VisualGraph   = 16;   // is there pseudographic (G1, G2, GT)
*/

#define VisualBold	1
#define VisualDim	2
#define VisualInverse	4
#define VisualColors	8
#define VisualGraph	16

/*
 *	Symbols 0 - 127 	assumed as Latin-1
 *		128 - 191 	assumed as pseudographic
 *		192 - 255	assumed as national
 *	This structure is equal with koi8 codetable, but at all
 *	allows describe any alphabet with symbols quantity <= 32
 *
 *	Structure of description file for transtaling chars (termcap entry Ct):
 *
 *	64 chars - translation national chars at output
 *	96 chars - translation Latin-1 chars (32 - 127) at national at input
 *	128 chars - translation up-half table at pseudographic or
 *		    national chars at input
 *	64 chars - translation pseudographic chars at output
 *
 *	- structure is compatible "down" with deco-usable
 *
 *	alghoritm for output:
 *	for national char :
 *	if Cs != 0, output Cs for switching terminal
 *	at national output mode and Ce to switching back
 *	for pseudographic char :
 *	if gs != 0, output gs for switching terminal at \
 *	pseudographic output mode and ge for switching back
 *
 *	for n char output outputTable[n]
 *
 */

/*
 * linedraw array contains chars for border drawing:
     0    horisontal line
     1    vertical line
     2    lower left corner
     3    lower center
     4    lower right corner
     5    left center
     6    center cross
     7    right center
     8    upper left corner
     9    upper center
     10   upper right corner
*/

#ifdef SCO_SYSV

#ifndef _STRUCT_WINSIZE
#define _STRUCT_WINSIZE
struct winsize {
	unsigned short ws_row;		/* rows, in characters*/
	unsigned short ws_col;		/* columns, in character */
	unsigned short ws_xpixel;	/* horizontal size, pixels */
	unsigned short ws_ypixel;	/* vertical size, pixels */
};
#endif

#endif

#endif
