/*-
 * Copyright (C) 1994-2000
 *  Engineering-Technical Company (R), All rights reserved.
 * Copyright (C) 2007-2013
 * Rashid N. "Citycat" Achilov		  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE Rashid "CityCat" Achilov ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This software based on sources 'yui' program, version 3.1.15,
 * developed in Engineering-Technological Company (http://www.itk.ru).
 * Authors of original code are Sergey Rozenfeld and Pavel Lazarev.
 * See COPYRIGHT.ITK for more details
 *
 *	$Id: date.c,v 3.2.2.4 2013/03/22 06:51:45 shelton Exp $
 */

#include <config.h>

#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
 #include <string.h>
#endif

#define  JULIAN_ADJUSTMENT    1721425L

static char bufDate[5];

static char *day_of_week[8] =
{
   "",
   "Sunday",
   "Monday",
   "Tuesday",
   "Wednesday",
   "Thursday",
   "Friday",
   "Saturday"
};

static char *month_of_year[13] =
{
   "",
   "January",
   "February",
   "March",
   "April",
   "May",
   "June",
   "July",
   "August",
   "September",
   "October",
   "November",
   "December"
};

static int month_total[] = {
	0, 0,  31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};

/*
 * Jan Feb Mar  Apr  May  Jun  Jul  Aug  Sep  Oct  Nov  Dec
 * 31  28  31   30   31   30   31   31   30   31   30   31
 */

int day_in_year(int year, int month, int day)
{
  /*  Returns */
  /*     >0   The day of the year starting from 1 */
  /*          Ex.    Jan 1, returns  1 */
  /*     -1   Illegal Date */
  int is_leap, month_days ;

  is_leap = ((year % 4 == 0 && year % 100 != 0)
  				|| year % 400 == 0) ?  1 : 0;

  month_days = month_total[month+1] - month_total[month];

  if (month == 2)
    month_days += is_leap ;

  if (year < 0 ||
	month < 1  ||  month > 12  ||
	day   < 1  ||  day   > month_days )
    return( -1 ) ;        			/* Illegal Date */

  if (month <= 2)  is_leap = 0;

  return(month_total[month] + day + is_leap);
}

long day_to_year(int yr)
{
  /*  Calculates the number of days to the year 		    */
  /*  This calculation takes into account the fact that 	    */
  /*     1)  Years divisible by 400 are always leap years. 	    */
  /*     2)  Years divisible by 100 but not 400 are not leap years. */
  /*     3)  Otherwise, years divisible by four are leap years.     */
  /*  Since we do not want to consider the current year, we will    */
  /*  subtract the year by 1 before doing the calculation. 	    */

  yr-- ;
  return(yr * 365L + yr / 4L - yr / 100L + yr / 400L);
}

long date_long( char *date_ptr )
{
  /*  Format: DDMMYYYY 							*/
  /*  Returns: 								*/
  /*   >0  -  Julian day 						*/
  /*          That is the number of days since the date  Jan 1, 4713 BC */
  /*          Ex.  Jan 1, 1981 is  2444606 				*/
  /*    0  -  NULL Date (dbf_date is all blank) 			*/
  /*   -1  -  Illegal Date 						*/

  int year, month, day, day_year;

  strcpy(bufDate, date_ptr + 4);
  year = atoi(bufDate);

  memcpy(bufDate, date_ptr + 2, 2);

  bufDate[2] = 0;
  month = atoi(bufDate);

  memcpy(bufDate, date_ptr, 2);

  bufDate[2] = 0;
  day = atoi(bufDate);

  day_year = day_in_year(year, month, day);

  if (day_year < 1)    			/* Illegal Date */
    return -1L ;

  return day_to_year(year) + day_year + JULIAN_ADJUSTMENT;
}

int date4dow(char *date_ptr)
{
  long date ;

  date = date_long(date_ptr) ;

  if (date < 0)
    return 0;

  return (int)((date + 1) % 7) + 1;
}

char *day(char *date_ptr)
{
  return day_of_week[date4dow(date_ptr)];
}

int date4month(char *date_ptr)
{
  memcpy(bufDate, date_ptr+2, 2);

  bufDate[2] = 0;
  return atoi(bufDate);
}

char *month(char *date_ptr)
{
  return month_of_year[date4month(date_ptr)];
}

#ifdef EXAMPLE
/* Generate main program to test these functions */

#include <stdio.h>
char buf[128];

int main()
{
  while( 1 ) {
      puts( "Enter date (DDMMYYYY) or 'q' to quit:" );
      gets( buf );
      if ( buf[0] == 'q' || buf[0] == 'Q' )
	  break;
      if ( 0 < date_long( buf ) )
	  printf( "It is %s, %s\n\n", day(buf), month(buf) );
      else
	  puts( "Illegal date.\n" );
  }
  return 0;
};

#endif
